/*============================================================================
  File:     FilteredIndexesOnSparseColumns.sql

  Summary:  Setup and investigate the power of Filtered Indexes
			on sparse columns.

  Date:     August 2008

  SQL Server Version: 10.0.1600.22 (SQL Server 2008 RTM)
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp & Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended as a supplement to the SQL Server 2008 Jumpstart or
  Metro training.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- This demo uses the SparseColumnTest database covered in the sparse
-- column section. For this demo, restore the database from the 
-- backup. The restore will take approx 1.5 minutes.

RESTORE DATABASE SparseColumnsTest FROM
DISK = 'C:\Metro Demos\Sparse Columns Demos\SparseColumns.bak'
WITH REPLACE, STATS = 2;
GO

-- Two tables have been pre-created
--		NonSparseDocRepository and SparseDocRepository
-- Each has 100000 rows of document metadata. There are a
-- few common column values for each document (e.g. name,
-- type) and then 20 columns that are distinct to each
-- type - stored in columns c20-c39 for doc-type 1,
-- c40-c59 for doc-type 2 etc
-- Although these are stored as integers, they
-- could be various types.
-- Each table has a clustered index on DocID.

Use SparseColumnsTest;
GO

-- Examine the row lengths and page counts (*big* tables!)
SELECT 'NonSparseDocRepository' AS TableName
	, Index_id AS IndexID
	, [avg_record_size_in_bytes]
	, [page_count]
	, *
FROM sys.dm_db_index_physical_stats (
	DB_ID ('SparseColumnsTest'),
	OBJECT_ID ('NonSparseDocRepository'),
	NULL, NULL, 'SAMPLED')
UNION ALL
SELECT 'SparseColumnsTest'
	, Index_id AS IndexID
	, [avg_record_size_in_bytes]
	, [page_count]
	, *
FROM sys.dm_db_index_physical_stats (
	DB_ID ('SparseColumnsTest'),
	OBJECT_ID ('SparseDocRepository'),
	NULL, NULL, 'SAMPLED');
GO

-- Let's create indexes over column c334 - with and without filters
-- against regular and sparse columns... 
-- is the index on sparse columns sparse?

CREATE NONCLUSTERED INDEX testc334 
ON SparseDocRepository(c334)

CREATE NONCLUSTERED INDEX testc334withfilter
ON SparseDocRepository(c334)
WHERE(c334 IS NOT NULL)

CREATE NONCLUSTERED INDEX testc334 
ON NonSparseDocRepository(c334)

CREATE NONCLUSTERED INDEX testc334withfilter
ON NonSparseDocRepository(c334)
WHERE(c334 IS NOT NULL)
GO

-- How many rows have a value for column c334?
SELECT COUNT(*) FROM NonSparseDocRepository
WHERE c334 IS NOT NULL

SELECT COUNT(*) FROM SparseDocRepository
WHERE c334 IS NOT NULL
GO

-- Examine the row lengths and page counts (*big* tables!)
-- From the output here you will see that the nonclustered
-- indexes are the SAME for sparse columns as they are when
-- the columns are not sparse. Sparse columns (if/when indexed)
-- should ALWAYS use a filter (WHERE SparseCol IS NOT NULL)!!
SELECT 'NonSparseDocRepository' AS TableName
	, Index_id AS IndexID
	, [avg_record_size_in_bytes]
	, [page_count]
	, *
FROM sys.dm_db_index_physical_stats (
	DB_ID ('SparseColumnsTest'),
	OBJECT_ID ('NonSparseDocRepository'),
	NULL, NULL, 'SAMPLED')
UNION ALL
SELECT 'SparseColumnsTest'
	, Index_id AS IndexID
	, [avg_record_size_in_bytes]
	, [page_count]
	, *
FROM sys.dm_db_index_physical_stats (
	DB_ID ('SparseColumnsTest'),
	OBJECT_ID ('SparseDocRepository'),
	NULL, NULL, 'SAMPLED');
GO

-----------------------------------------------------------
------------------Extra Stuff -----------------------------
-----------------------------------------------------------
-- You can show some queries and/or statistics but the key
-- points have been shown!
SELECT * FROM SparseDocRepository
WHERE c334 IS NOT NULL

SELECT * FROM NonSparseDocRepository
WHERE c334 IS NOT NULL

SELECT * FROM SparseDocRepository
WHERE c334 = 28668

SELECT * FROM NonSparseDocRepository
WHERE c334 = 28668

DBCC SHOW_STATISTICS ('SparseDocRepository', 'testc334')
DBCC SHOW_STATISTICS ('SparseDocRepository', 'testc334withfilter')

DBCC SHOW_STATISTICS ('NonSparseDocRepository', 'testc334')
DBCC SHOW_STATISTICS ('NonSparseDocRepository', 'testc334withfilter')